<?xml version="1.0"?>
<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:doc="http://www.jenitennison.com/xslt/doc"
                extension-element-prefixes="doc"
                doc:base="summary.xsl">

<xsl:import href="xslt-doc-utils.xsl" />
<xsl:import href="table.xsl" />

<xsl:param name="tables-rtf">
   <xsl:copy-of select="$prefs/summary/tables" />
</xsl:param>

<xsl:template name="add-parameters">
	XSLTProcessor.addParameter('source-stylesheet', '<xsl:call-template name="escape-js">
                                                       <xsl:with-param name="string" select="$source-stylesheet" />
                                                    </xsl:call-template>');
   XSLTProcessor.addParameter('current-stylesheet', '<xsl:call-template name="escape-js">
                                                       <xsl:with-param name="string" select="$current-stylesheet" />
                                                     </xsl:call-template>');
   XSLTProcessor.addParameter('show', '<xsl:value-of select="$show" />');
</xsl:template>

<xsl:variable name="row">
   <xsl:text>row</xsl:text>
   <xsl:choose>
      <xsl:when test="generate-id($node) = generate-id($node-construct)"><xsl:value-of select="$show" /></xsl:when>
      <xsl:otherwise><xsl:value-of select="count($node-construct/preceding::* | $node-construct/ancestor::*) + 1" /></xsl:otherwise>
   </xsl:choose>
</xsl:variable>

<xsl:template match="xsl:stylesheet">
   <html>
      <head>
         <link rel="stylesheet" type="text/css" href="summary.css" />
         <xsl:call-template name="insert-script-for-tables">
            <xsl:with-param name="xml-file" select="$current-stylesheet" />
            <xsl:with-param name="xsl-file" select="'summary.xsl'" />
         </xsl:call-template>
         <script type="text/javascript">
            var selectedRow = '';
            function select(row) {
               if (selectedRow != '') {
                  selectedRow.style.background = '';
                  selectedRow.style.color = '';
                  selectedRow.style.cursor = '';
               }
               selectedRow = row;
               row.style.background = '<xsl:value-of select="$prefs/summary/selected/@background" />';
               row.style.color = '<xsl:value-of select="$prefs/summary/selected/@color" />';
               row.style.cursor = '<xsl:value-of select="$prefs/summary/selected/@cursor" />';
            }
         </script>
      </head>
      <body onload="javascript:select({$row});">
         <xsl:call-template name="insert-table">
            <xsl:with-param name="name" select="'import'" />
            <xsl:with-param name="rows" select="xsl:import | xsl:include" />
         </xsl:call-template>
         
         <xsl:call-template name="insert-table">
            <xsl:with-param name="name" select="'spaces'" />
            <xsl:with-param name="rows" select="xsl:strip-space | xsl:preserve-space" />
         </xsl:call-template>
         
         <xsl:call-template name="insert-table">
            <xsl:with-param name="name" select="'output'" />
            <xsl:with-param name="rows" select="xsl:output" />
         </xsl:call-template>
         
         <xsl:call-template name="insert-table">
            <xsl:with-param name="name" select="'namespaces'" />
            <xsl:with-param name="rows" select="xsl:namespace-alias" />
         </xsl:call-template>
         
         <xsl:call-template name="insert-table">
            <xsl:with-param name="name" select="'variables'" />
            <xsl:with-param name="rows" select="xsl:variable | xsl:param" />
         </xsl:call-template>
         
         <xsl:call-template name="insert-table">
            <xsl:with-param name="name" select="'keys'" />
            <xsl:with-param name="rows" select="xsl:key" />
         </xsl:call-template>
         
         <xsl:call-template name="insert-table">
            <xsl:with-param name="name" select="'templates'" />
            <xsl:with-param name="rows" select="xsl:template[not(@name)]" />
         </xsl:call-template>
         
         <xsl:call-template name="insert-table">
            <xsl:with-param name="name" select="'named-templates'" />
            <xsl:with-param name="rows" select="xsl:template[@name]" />
         </xsl:call-template>
         
         <xsl:call-template name="insert-table">
            <xsl:with-param name="name" select="'attributes'" />
            <xsl:with-param name="rows" select="xsl:attribute-set" />
         </xsl:call-template>
         
         <xsl:call-template name="insert-table">
            <xsl:with-param name="name" select="'decimals'" />
            <xsl:with-param name="rows" select="xsl:decimal-format" />
         </xsl:call-template>
      </body>
   </html>
</xsl:template>

<xsl:template match="*" mode="row">
   <xsl:param name="table" select="/.." />
   <xsl:variable name="id" select="@original-position" />
   <xsl:variable name="override">
      <xsl:if test="@name">
         <xsl:apply-templates select="." mode="find-overriding" />
      </xsl:if>
   </xsl:variable>
	<tr id="row{$id}">
      <xsl:attribute name="onclick">
         <xsl:text>javascript:top.loadSource('</xsl:text>
         <xsl:call-template name="escape-js">
            <xsl:with-param name="string" select="$current-stylesheet" />
         </xsl:call-template>
         <xsl:text>', '</xsl:text>
         <xsl:value-of select="$id" />
         <xsl:text>'); select(row</xsl:text>
         <xsl:value-of select="$id" />
         <xsl:text>)</xsl:text>
      </xsl:attribute>
      <xsl:variable name="node" select="." />
      <xsl:for-each select="$table/col">
         <xsl:variable name="value">
            <xsl:apply-templates select="$node" mode="cell-value">
               <xsl:with-param name="cell" select="@value" />
            </xsl:apply-templates>
         </xsl:variable>
         <td>
            <xsl:if test="string($override)">
               <xsl:attribute name="class">overridden</xsl:attribute>
            </xsl:if>
            <xsl:choose>
               <xsl:when test="string($value)"><xsl:copy-of select="$value" /></xsl:when>
               <xsl:otherwise>&#160;</xsl:otherwise>
            </xsl:choose>
         </td>
      </xsl:for-each>
   </tr>
</xsl:template>

</xsl:stylesheet>