<?xml version="1.0"?>
<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:param name="source-stylesheet" />
<xsl:param name="current-stylesheet" select="$source-stylesheet" />
<xsl:param name="show" select="2" />

<xsl:key name="stylesheet-elements" match="*" use="count(preceding::* | ancestor::*) + 1" />

<xsl:variable name="node" select="key('stylesheet-elements', $show)" />
<xsl:variable name="node-construct" select="$node/ancestor-or-self::*[parent::xsl:stylesheet]" />

<xsl:variable name="prefs" select="document('xslt-doc-prefs.xml')/prefs" />

<xsl:template name="substring-before-last">
	<xsl:param name="string" />
   <xsl:param name="find" />
   <xsl:if test="contains($string, $find)">
      <xsl:value-of select="substring-before($string, $find)" />
      <xsl:value-of select="$find" />
      <xsl:call-template name="substring-before-last">
         <xsl:with-param name="string" select="substring-after($string, $find)" />
         <xsl:with-param name="find" select="$find" />
      </xsl:call-template>
   </xsl:if>
</xsl:template>

<xsl:key name="named-constructs" match="xsl:stylesheet/xsl:*[@name]" use="@name" />

<xsl:template match="xsl:stylesheet" mode="find-construct">
   <xsl:param name="name" />
   <xsl:param name="type" />
   <xsl:param name="stylesheet" select="$source-stylesheet" />
   <xsl:variable name="construct" select="key('named-constructs', $name)[contains($type, name())]" />
   <xsl:choose>
      <xsl:when test="$construct">
         <in><xsl:value-of select="$stylesheet" /></in>
         <id><xsl:value-of select="count($construct/preceding::* | $construct/ancestor::*) + 1" /></id>
      </xsl:when>
      <xsl:otherwise>
         <xsl:apply-templates select="*[self::xsl:import or self::xsl:include][1]" mode="find-construct">
            <xsl:with-param name="name" select="$name" />
            <xsl:with-param name="type" select="$type" />
            <xsl:with-param name="stylesheet" select="$stylesheet" />
         </xsl:apply-templates>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="xsl:import | xsl:include" mode="find-construct">
   <xsl:param name="name" />
   <xsl:param name="type" />
   <xsl:param name="stylesheet" />
   <xsl:variable name="construct">
      <xsl:apply-templates select="document(@href)/xsl:stylesheet" mode="find-construct">
         <xsl:with-param name="name" select="$name" />
         <xsl:with-param name="type" select="$type" />
         <xsl:with-param name="stylesheet">
            <xsl:call-template name="resolve">
               <xsl:with-param name="filename" select="@href" />
               <xsl:with-param name="base-URI" select="$stylesheet" />
            </xsl:call-template>
         </xsl:with-param>
      </xsl:apply-templates>
   </xsl:variable>
   <xsl:choose>
      <xsl:when test="string($construct)">
         <xsl:copy-of select="$construct" />
      </xsl:when>
      <xsl:otherwise>
         <xsl:variable name="next" select="following-sibling::*[self::xsl:import or self::xsl:include][1]" />
         <xsl:apply-templates select="$next" mode="find-construct">
            <xsl:with-param name="name" select="$name" />
            <xsl:with-param name="type" select="$type" />
            <xsl:with-param name="stylesheet" select="$stylesheet" />
         </xsl:apply-templates>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="xsl:*[@name]" mode="find-overriding">
   <xsl:variable name="name" select="@name" />
   <xsl:variable name="type" select="name()" />
   <xsl:choose>
      <xsl:when test="following-sibling::*[name() = $type][@name = $name][1]">
         <xsl:value-of select="$current-stylesheet" />
      </xsl:when>
      <xsl:when test="$current-stylesheet != $source-stylesheet">
         <xsl:apply-templates select="document($source-stylesheet)/xsl:stylesheet" mode="find-overriding">
            <xsl:with-param name="name" select="$name" />
            <xsl:with-param name="type" select="$type" />
         </xsl:apply-templates>
      </xsl:when>
   </xsl:choose>
</xsl:template>

<xsl:template match="xsl:stylesheet" mode="find-overriding">
   <xsl:param name="name" />
   <xsl:param name="type" />
   <xsl:param name="stylesheet" select="$source-stylesheet" />
   <xsl:variable name="construct" select="key('named-constructs', $name)[name() = $type]" />
   <xsl:choose>
      <xsl:when test="$construct">
         <in><xsl:value-of select="$stylesheet" /></in>
         <id><xsl:value-of select="count($construct/preceding::* | $construct/ancestor::*) + 1" /></id>
      </xsl:when>
      <xsl:otherwise>
         <xsl:variable name="next" select="(xsl:import | xsl:include)[1]" />
         <xsl:variable name="next-URI">
            <xsl:call-template name="resolve">
               <xsl:with-param name="filename" select="$next/@href" />
               <xsl:with-param name="base-URI" select="$stylesheet" />
            </xsl:call-template>
         </xsl:variable>
         <xsl:if test="$next-URI != $current-stylesheet">
            <xsl:apply-templates select="$next" mode="find-overriding">
               <xsl:with-param name="name" select="$name" />
               <xsl:with-param name="type" select="$type" />
               <xsl:with-param name="stylesheet" select="$next-URI" />
            </xsl:apply-templates>
         </xsl:if>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="xsl:import | xsl:include" mode="find-overriding">
   <xsl:param name="name" />
   <xsl:param name="type" />
   <xsl:param name="stylesheet" />
   <xsl:variable name="construct">
      <xsl:apply-templates select="document(@href)/xsl:stylesheet" mode="find-overriding">
         <xsl:with-param name="name" select="$name" />
         <xsl:with-param name="type" select="$type" />
         <xsl:with-param name="stylesheet" select="$stylesheet" />
      </xsl:apply-templates>
   </xsl:variable>
   <xsl:choose>
      <xsl:when test="string($construct)">
         <xsl:copy-of select="$construct" />
      </xsl:when>
      <xsl:otherwise>
         <xsl:variable name="next" select="(following-sibling::xsl:import | following-sibling::xsl:include)[1]" />
         <xsl:variable name="next-URI">
            <xsl:call-template name="resolve">
               <xsl:with-param name="filename" select="$next/@href" />
               <xsl:with-param name="base-URI" select="$stylesheet" />
            </xsl:call-template>
         </xsl:variable>
         <xsl:if test="$next-URI != $current-stylesheet">
            <xsl:apply-templates select="$next" mode="find-overriding">
               <xsl:with-param name="name" select="$name" />
               <xsl:with-param name="type" select="$type" />
               <xsl:with-param name="stylesheet" select="$stylesheet" />
            </xsl:apply-templates>
         </xsl:if>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template name="escape-js">
   <xsl:param name="string" />
   <xsl:variable name="quotes-escaped">
      <xsl:call-template name="js-escape-char">
         <xsl:with-param name="string" select="$string" />
         <xsl:with-param name="char" select='"&apos;"' />
      </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="backslash-escaped">
      <xsl:call-template name="js-escape-char">
         <xsl:with-param name="string" select="$quotes-escaped" />
         <xsl:with-param name="char" select="'\'" />
      </xsl:call-template>
   </xsl:variable>
   <xsl:call-template name="js-escape-line-breaks">
      <xsl:with-param name="string" select="$backslash-escaped" />
   </xsl:call-template>
</xsl:template>

<xsl:template name="js-escape-line-breaks">
   <xsl:param name="string" />
   <xsl:choose>
      <xsl:when test="contains($string, '&#x0A;')">
         <xsl:value-of select="substring-before($string, '&#x0A;')" />
         <xsl:text>\n</xsl:text>
         <xsl:call-template name="js-escape-line-breaks">
           <xsl:with-param name="string" select="substring-after($string, '&#x0A;')" />
         </xsl:call-template>
      </xsl:when>
      <xsl:otherwise><xsl:value-of select="$string" /></xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template name="js-escape-char">
   <xsl:param name="string" />
   <xsl:param name="char" />
   <xsl:choose>
      <xsl:when test='contains($string, $char)'>
         <xsl:value-of select='substring-before($string, $char)' />
         <xsl:text />\<xsl:value-of select="$char" />
         <xsl:call-template name="js-escape-char">
           <xsl:with-param name="string" select='substring-after($string, $char)' />
           <xsl:with-param name="char" select='$char' />
         </xsl:call-template>
      </xsl:when>
      <xsl:otherwise><xsl:value-of select="$string" /></xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template name="resolve">
   <xsl:param name="filename" />
   <xsl:param name="base-URI" />
   <xsl:choose>
      <xsl:when test="starts-with($filename, 'http:') or 
                      starts-with($filename, 'file:') or
                      substring($filename, 2, 2) = ':\'">
         <xsl:value-of select="$filename" />
      </xsl:when>
      <xsl:when test="starts-with($filename, '/') or
                      starts-with($filename, '\')">
         <xsl:choose>
            <xsl:when test="starts-with($base-URI, 'http:')">
               <xsl:text>http://</xsl:text>
               <xsl:value-of select="substring-before(substring($base-URI, 8), '/')" />
            </xsl:when>
            <xsl:when test="starts-with($base-URI, 'file:')">
               <xsl:text>file:///</xsl:text>
               <xsl:value-of select="substring-before(substring($base-URI, 9), '/')" />
            </xsl:when>
            <xsl:otherwise><xsl:value-of select="substring($base-URI, 1, 2)" /></xsl:otherwise>
         </xsl:choose>
         <xsl:value-of select="$filename" />
      </xsl:when>
      <xsl:otherwise>
         <xsl:call-template name="base-URI">
            <xsl:with-param name="file" select="$base-URI" />
         </xsl:call-template>
         <xsl:value-of select="$filename" />
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template name="base-URI">
   <xsl:param name="file" />
   <xsl:if test="starts-with($file, 'http:') or
                      starts-with($file, 'file:') or
                      substring($file, 2, 2) = ':\'">
      <xsl:call-template name="getDir">
         <xsl:with-param name="string" select="$file" />
      </xsl:call-template>
   </xsl:if>
</xsl:template>

<xsl:template name="getDir">
   <xsl:param name="string" />
   <xsl:choose>
      <xsl:when test="substring($string, string-length($string)) = '/' or
                      substring($string, string-length($string)) = '\'">
         <xsl:value-of select="$string" />
      </xsl:when>
      <xsl:otherwise>
         <xsl:call-template name="getDir">
            <xsl:with-param name="string" select="substring($string, 1, string-length($string) - 1)" />
         </xsl:call-template>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

</xsl:stylesheet>